<!DOCTYPE html>
<html>
  <head>
    <title>{{name}}</title>
    {{>head}}
  </head>
  <body>
    <div data-role="page">
      <div data-role="header">
        {{>home}}
        <h1>{{name}}</h1>
      </div>
      <div role="main" class="ui-content">
        <h1>{{name}}</h1>
        {{#description}}
        {{{description}}}
        {{/description}}
        <div data-role="controlgroup" data-type="horizontal" data-mini="true">
          {{#available_formats}}
          <a href="{{url}}.{{.}}{{url_params}}" rel="external" class="ui-btn ui-corner-all">
            {{.}}
          </a>
          {{/available_formats}}
        </div>
        {{#wq_chart_type}}
        <hr>
        <svg data-wq-url="{{url}}.csv{{url_params}}" data-wq-type="{{wq_chart_type}}"></svg>
        {{/wq_chart_type}}
        <hr>
        {{{table}}}
      </div>
    </div>
  </body>
</html>
